:root {
  --suffolk-blue: #15284b;
  --suffolk-blue-rgb: 21, 40, 75;
  --suffolk-gold: #fdb924;
  --suffolk-gold-rgb: 253, 185, 36;
}

.navbar-nav {
  --bs-nav-link-color: rgba(255, 255, 255, .65);
  --bs-nav-link-hover-color: rgba(255, 255, 255, .65);
}